@@ -1,16 +1,20 @@
 use ibc_proto::ibc::core::commitment::v1::MerkleProof;
 
-use crate::ics02_client::client_consensus::AnyConsensusState;
-use crate::ics02_client::client_def::ClientDef;
-use crate::ics02_client::client_state::AnyClientState;
-use crate::ics02_client::error::Error;
-use crate::ics03_connection::connection::ConnectionEnd;
-use crate::ics04_channel::channel::ChannelEnd;
-use crate::ics04_channel::packet::Sequence;
-use crate::ics23_commitment::commitment::{CommitmentPrefix, CommitmentProofBytes, CommitmentRoot};
-use crate::ics23_commitment::merkle::apply_prefix;
-use crate::ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId};
-use crate::ics24_host::Path;
+use crate::core::ics02_client::client_consensus::AnyConsensusState;
+use crate::core::ics02_client::client_def::ClientDef;
+use crate::core::ics02_client::client_state::AnyClientState;
+use crate::core::ics02_client::context::ClientReader;
+use crate::core::ics02_client::error::Error;
+use crate::core::ics03_connection::connection::ConnectionEnd;
+use crate::core::ics04_channel::channel::ChannelEnd;
+use crate::core::ics04_channel::context::ChannelReader;
+use crate::core::ics04_channel::packet::Sequence;
+use crate::core::ics23_commitment::commitment::{
+    CommitmentPrefix, CommitmentProofBytes, CommitmentRoot,
+};
+use crate::core::ics23_commitment::merkle::apply_prefix;
+use crate::core::ics24_host::identifier::{ChannelId, ClientId, ConnectionId, PortId};
+use crate::core::ics24_host::Path;
 use crate::mock::client_state::{MockClientState, MockConsensusState};
 use crate::mock::header::MockHeader;
 use crate::prelude::*;
@@ -26,6 +30,8 @@ impl ClientDef for MockClient {
 
     fn check_header_and_update_state(
         &self,
+        _ctx: &dyn ClientReader,
+        _client_id: ClientId,
         client_state: Self::ClientState,
         header: Self::Header,
     ) -> Result<(Self::ClientState, Self::ConsensusState), Error> {
@@ -35,34 +41,33 @@ impl ClientDef for MockClient {
                 client_state.latest_height(),
             ));
         }
-        Ok((MockClientState(header), MockConsensusState::new(header)))
+        Ok((
+            MockClientState::new(header),
+            MockConsensusState::new(header),
+        ))
     }
 
     fn verify_client_consensus_state(
         &self,
         _client_state: &Self::ClientState,
-        height: Height,
+        _height: Height,
         prefix: &CommitmentPrefix,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         client_id: &ClientId,
-        _consensus_height: Height,
+        consensus_height: Height,
         _expected_consensus_state: &AnyConsensusState,
     ) -> Result<(), Error> {
         let client_prefixed_path = Path::ClientConsensusState {
             client_id: client_id.clone(),
-            epoch: height.revision_number,
-            height: height.revision_height,
+            epoch: consensus_height.revision_number,
+            height: consensus_height.revision_height,
         }
         .to_string();
 
         let _path =
             apply_prefix(prefix, vec![client_prefixed_path]).map_err(Error::empty_prefix)?;
 
-        // TODO - add ctx to all client verification functions
-        // let cs = ctx.fetch_self_consensus_state(height);
-        // TODO - implement this
-        // proof.verify_membership(cs.root(), path, expected_consensus_state)
-
         Ok(())
     }
 
@@ -72,7 +77,8 @@ impl ClientDef for MockClient {
         _height: Height,
         _prefix: &CommitmentPrefix,
         _proof: &CommitmentProofBytes,
-        _connection_id: Option<&ConnectionId>,
+        _root: &CommitmentRoot,
+        _connection_id: &ConnectionId,
         _expected_connection_end: &ConnectionEnd,
     ) -> Result<(), Error> {
         Ok(())
@@ -84,6 +90,7 @@ impl ClientDef for MockClient {
         _height: Height,
         _prefix: &CommitmentPrefix,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         _port_id: &PortId,
         _channel_id: &ChannelId,
         _expected_channel_end: &ChannelEnd,
@@ -95,10 +102,10 @@ impl ClientDef for MockClient {
         &self,
         _client_state: &Self::ClientState,
         _height: Height,
-        _root: &CommitmentRoot,
         _prefix: &CommitmentPrefix,
-        _client_id: &ClientId,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
+        _client_id: &ClientId,
         _expected_client_state: &AnyClientState,
     ) -> Result<(), Error> {
         Ok(())
@@ -106,53 +113,66 @@ impl ClientDef for MockClient {
 
     fn verify_packet_data(
         &self,
+        _ctx: &dyn ChannelReader,
         _client_state: &Self::ClientState,
         _height: Height,
+        _connection_end: &ConnectionEnd,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         _port_id: &PortId,
         _channel_id: &ChannelId,
-        _seq: &Sequence,
-        _data: String,
+        _sequence: Sequence,
+        _commitment: String,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     fn verify_packet_acknowledgement(
         &self,
+        _ctx: &dyn ChannelReader,
         _client_state: &Self::ClientState,
         _height: Height,
+        _connection_end: &ConnectionEnd,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         _port_id: &PortId,
         _channel_id: &ChannelId,
-        _seq: &Sequence,
-        _data: Vec<u8>,
+        _sequence: Sequence,
+        _ack: Vec<u8>,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     fn verify_next_sequence_recv(
         &self,
+        _ctx: &dyn ChannelReader,
         _client_state: &Self::ClientState,
         _height: Height,
+        _connection_end: &ConnectionEnd,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         _port_id: &PortId,
         _channel_id: &ChannelId,
-        _seq: &Sequence,
+        _sequence: Sequence,
     ) -> Result<(), Error> {
         Ok(())
     }
 
     fn verify_packet_receipt_absence(
         &self,
+        _ctx: &dyn ChannelReader,
         _client_state: &Self::ClientState,
         _height: Height,
+        _connection_end: &ConnectionEnd,
         _proof: &CommitmentProofBytes,
+        _root: &CommitmentRoot,
         _port_id: &PortId,
         _channel_id: &ChannelId,
-        _seq: &Sequence,
+        _sequence: Sequence,
     ) -> Result<(), Error> {
         Ok(())
     }
+
     fn verify_upgrade_and_update_state(
         &self,
         client_state: &Self::ClientState,
