@@ -1,44 +1,48 @@
 //! Implementation of a global context mock. Used in testing handlers of all IBC modules.
 
 use crate::prelude::*;
-use alloc::collections::btree_map::BTreeMap as HashMap;
+
+use alloc::collections::btree_map::BTreeMap;
 use core::cmp::min;
 
+use tracing::debug;
+
 use prost_types::Any;
 use sha2::Digest;
 
-use crate::application::ics20_fungible_token_transfer::context::Ics20Context;
+use crate::applications::ics20_fungible_token_transfer::context::Ics20Context;
+use crate::clients::ics07_tendermint::client_state::test_util::get_dummy_tendermint_client_state;
+use crate::core::ics02_client::client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight};
+use crate::core::ics02_client::client_state::AnyClientState;
+use crate::core::ics02_client::client_type::ClientType;
+use crate::core::ics02_client::context::{ClientKeeper, ClientReader};
+use crate::core::ics02_client::error::Error as Ics02Error;
+use crate::core::ics02_client::header::AnyHeader;
+use crate::core::ics03_connection::connection::ConnectionEnd;
+use crate::core::ics03_connection::context::{ConnectionKeeper, ConnectionReader};
+use crate::core::ics03_connection::error::Error as Ics03Error;
+use crate::core::ics04_channel::channel::ChannelEnd;
+use crate::core::ics04_channel::context::{ChannelKeeper, ChannelReader};
+use crate::core::ics04_channel::error::Error as Ics04Error;
+use crate::core::ics04_channel::packet::{Receipt, Sequence};
+use crate::core::ics05_port::capabilities::Capability;
+use crate::core::ics05_port::context::PortReader;
+use crate::core::ics05_port::error::Error as Ics05Error;
+use crate::core::ics23_commitment::commitment::CommitmentPrefix;
+use crate::core::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
+use crate::core::ics26_routing::context::Ics26Context;
+use crate::core::ics26_routing::handler::{deliver, dispatch};
+use crate::core::ics26_routing::msgs::Ics26Envelope;
 use crate::events::IbcEvent;
-use crate::ics02_client::client_consensus::{AnyConsensusState, AnyConsensusStateWithHeight};
-use crate::ics02_client::client_state::AnyClientState;
-use crate::ics02_client::client_type::ClientType;
-use crate::ics02_client::context::{ClientKeeper, ClientReader};
-use crate::ics02_client::error::Error as Ics02Error;
-use crate::ics02_client::header::AnyHeader;
-use crate::ics03_connection::connection::ConnectionEnd;
-use crate::ics03_connection::context::{ConnectionKeeper, ConnectionReader};
-use crate::ics03_connection::error::Error as Ics03Error;
-use crate::ics04_channel::channel::ChannelEnd;
-use crate::ics04_channel::context::{ChannelKeeper, ChannelReader};
-use crate::ics04_channel::error::Error as Ics04Error;
-use crate::ics04_channel::packet::{Receipt, Sequence};
-use crate::ics05_port::capabilities::Capability;
-use crate::ics05_port::context::PortReader;
-use crate::ics05_port::error::Error as Ics05Error;
-use crate::ics07_tendermint::client_state::test_util::get_dummy_tendermint_client_state;
-use crate::ics18_relayer::context::Ics18Context;
-use crate::ics18_relayer::error::Error as Ics18Error;
-use crate::ics23_commitment::commitment::CommitmentPrefix;
-use crate::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
-use crate::ics26_routing::context::Ics26Context;
-use crate::ics26_routing::handler::{deliver, dispatch};
-use crate::ics26_routing::msgs::Ics26Envelope;
 use crate::mock::client_state::{MockClientRecord, MockClientState, MockConsensusState};
 use crate::mock::header::MockHeader;
 use crate::mock::host::{HostBlock, HostType};
+use crate::relayer::ics18_relayer::context::Ics18Context;
+use crate::relayer::ics18_relayer::error::Error as Ics18Error;
 use crate::signer::Signer;
 use crate::timestamp::Timestamp;
 use crate::Height;
+use core::time::Duration;
 
 /// A context implementing the dependencies necessary for testing any IBC module.
 #[derive(Clone, Debug)]
@@ -63,49 +67,55 @@ pub struct MockContext {
     history: Vec<HostBlock>,
 
     /// The set of all clients, indexed by their id.
-    clients: HashMap<ClientId, MockClientRecord>,
+    clients: BTreeMap<ClientId, MockClientRecord>,
+
+    /// Tracks the processed time for clients header updates
+    client_processed_times: BTreeMap<(ClientId, Height), Timestamp>,
+
+    /// Tracks the processed height for the clients
+    client_processed_heights: BTreeMap<(ClientId, Height), Height>,
 
     /// Counter for the client identifiers, necessary for `increase_client_counter` and the
     /// `client_counter` methods.
     client_ids_counter: u64,
 
     /// Association between client ids and connection ids.
-    client_connections: HashMap<ClientId, ConnectionId>,
+    client_connections: BTreeMap<ClientId, ConnectionId>,
 
     /// All the connections in the store.
-    connections: HashMap<ConnectionId, ConnectionEnd>,
+    connections: BTreeMap<ConnectionId, ConnectionEnd>,
 
     /// Counter for connection identifiers (see `increase_connection_counter`).
     connection_ids_counter: u64,
 
     /// Association between connection ids and channel ids.
-    connection_channels: HashMap<ConnectionId, Vec<(PortId, ChannelId)>>,
+    connection_channels: BTreeMap<ConnectionId, Vec<(PortId, ChannelId)>>,
 
     /// Counter for channel identifiers (see `increase_channel_counter`).
     channel_ids_counter: u64,
 
     /// All the channels in the store. TODO Make new key PortId X ChanneId
-    channels: HashMap<(PortId, ChannelId), ChannelEnd>,
+    channels: BTreeMap<(PortId, ChannelId), ChannelEnd>,
 
     /// Tracks the sequence number for the next packet to be sent.
-    next_sequence_send: HashMap<(PortId, ChannelId), Sequence>,
+    next_sequence_send: BTreeMap<(PortId, ChannelId), Sequence>,
 
     /// Tracks the sequence number for the next packet to be received.
-    next_sequence_recv: HashMap<(PortId, ChannelId), Sequence>,
+    next_sequence_recv: BTreeMap<(PortId, ChannelId), Sequence>,
 
     /// Tracks the sequence number for the next packet to be acknowledged.
-    next_sequence_ack: HashMap<(PortId, ChannelId), Sequence>,
+    next_sequence_ack: BTreeMap<(PortId, ChannelId), Sequence>,
 
-    packet_acknowledgement: HashMap<(PortId, ChannelId, Sequence), String>,
+    packet_acknowledgement: BTreeMap<(PortId, ChannelId, Sequence), String>,
 
     /// Maps ports to their capabilities
-    port_capabilities: HashMap<PortId, Capability>,
+    port_capabilities: BTreeMap<PortId, Capability>,
 
     /// Constant-size commitments to packets data fields
-    packet_commitment: HashMap<(PortId, ChannelId, Sequence), String>,
+    packet_commitment: BTreeMap<(PortId, ChannelId, Sequence), String>,
 
     // Used by unordered channel
-    packet_receipt: HashMap<(PortId, ChannelId, Sequence), Receipt>,
+    packet_receipt: BTreeMap<(PortId, ChannelId, Sequence), Receipt>,
 }
 
 /// Returns a MockContext with bare minimum initialization: no clients, no connections and no channels are
@@ -168,6 +178,8 @@ impl MockContext {
             connections: Default::default(),
             client_ids_counter: 0,
             clients: Default::default(),
+            client_processed_times: Default::default(),
+            client_processed_heights: Default::default(),
             client_connections: Default::default(),
             channels: Default::default(),
             connection_channels: Default::default(),
@@ -209,7 +221,7 @@ impl MockContext {
         let (client_state, consensus_state) = match client_type {
             // If it's a mock client, create the corresponding mock states.
             ClientType::Mock => (
-                Some(MockClientState(MockHeader::new(client_state_height)).into()),
+                Some(MockClientState::new(MockHeader::new(client_state_height)).into()),
                 MockConsensusState::new(MockHeader::new(cs_height)).into(),
             ),
             // If it's a Tendermint client, we need TM states.
@@ -218,6 +230,7 @@ impl MockContext {
                     self.host_chain_id.clone(),
                     cs_height.revision_height,
                 );
+
                 let consensus_state = AnyConsensusState::from(light_block.clone());
                 let client_state =
                     get_dummy_tendermint_client_state(light_block.signed_header.header);
@@ -228,6 +241,8 @@ impl MockContext {
         };
         let consensus_states = vec![(cs_height, consensus_state)].into_iter().collect();
 
+        debug!("consensus states: {:?}", consensus_states);
+
         let client_record = MockClientRecord {
             client_type,
             client_state,
@@ -237,6 +252,72 @@ impl MockContext {
         self
     }
 
+    pub fn with_client_parametrized_history(
+        mut self,
+        client_id: &ClientId,
+        client_state_height: Height,
+        client_type: Option<ClientType>,
+        consensus_state_height: Option<Height>,
+    ) -> Self {
+        let cs_height = consensus_state_height.unwrap_or(client_state_height);
+        let prev_cs_height = cs_height.clone().sub(1).unwrap_or(client_state_height);
+
+        let client_type = client_type.unwrap_or(ClientType::Mock);
+
+        let (client_state, consensus_state) = match client_type {
+            // If it's a mock client, create the corresponding mock states.
+            ClientType::Mock => (
+                Some(MockClientState::new(MockHeader::new(client_state_height)).into()),
+                MockConsensusState::new(MockHeader::new(cs_height)).into(),
+            ),
+            // If it's a Tendermint client, we need TM states.
+            ClientType::Tendermint => {
+                let light_block = HostBlock::generate_tm_block(
+                    self.host_chain_id.clone(),
+                    cs_height.revision_height,
+                );
+
+                let consensus_state = AnyConsensusState::from(light_block.clone());
+                let client_state =
+                    get_dummy_tendermint_client_state(light_block.signed_header.header);
+
+                // Return the tuple.
+                (Some(client_state), consensus_state)
+            }
+        };
+
+        let prev_consensus_state = match client_type {
+            // If it's a mock client, create the corresponding mock states.
+            ClientType::Mock => MockConsensusState::new(MockHeader::new(prev_cs_height)).into(),
+            // If it's a Tendermint client, we need TM states.
+            ClientType::Tendermint => {
+                let light_block = HostBlock::generate_tm_block(
+                    self.host_chain_id.clone(),
+                    prev_cs_height.revision_height,
+                );
+                AnyConsensusState::from(light_block)
+            }
+        };
+
+        let consensus_states = vec![
+            (prev_cs_height, prev_consensus_state),
+            (cs_height, consensus_state),
+        ]
+        .into_iter()
+        .collect();
+
+        debug!("consensus states: {:?}", consensus_states);
+
+        let client_record = MockClientRecord {
+            client_type,
+            client_state,
+            consensus_states,
+        };
+
+        self.clients.insert(client_id.clone(), client_record);
+        self
+    }
+
     /// Associates a connection to this context.
     pub fn with_connection(
         mut self,
@@ -347,7 +428,7 @@ impl MockContext {
 
     /// Accessor for a block of the local (host) chain from this context.
     /// Returns `None` if the block at the requested height does not exist.
-    fn host_block(&self, target_height: Height) -> Option<&HostBlock> {
+    pub fn host_block(&self, target_height: Height) -> Option<&HostBlock> {
         let target = target_height.revision_height as usize;
         let latest = self.latest_height.revision_height as usize;
 
@@ -431,6 +512,21 @@ impl MockContext {
             })
             .collect()
     }
+
+    pub fn latest_client_states(&self, client_id: &ClientId) -> &AnyClientState {
+        self.clients[client_id].client_state.as_ref().unwrap()
+    }
+
+    pub fn latest_consensus_states(
+        &self,
+        client_id: &ClientId,
+        height: &Height,
+    ) -> &AnyConsensusState {
+        self.clients[client_id]
+            .consensus_states
+            .get(height)
+            .unwrap()
+    }
 }
 
 impl Ics26Context for MockContext {}
@@ -578,9 +674,47 @@ impl ChannelReader for MockContext {
         self.timestamp
     }
 
+    fn client_update_time(
+        &self,
+        client_id: &ClientId,
+        height: Height,
+    ) -> Result<Timestamp, Ics04Error> {
+        match self
+            .client_processed_times
+            .get(&(client_id.clone(), height))
+        {
+            Some(time) => Ok(*time),
+            None => Err(Ics04Error::processed_time_not_found(
+                client_id.clone(),
+                height,
+            )),
+        }
+    }
+
+    fn client_update_height(
+        &self,
+        client_id: &ClientId,
+        height: Height,
+    ) -> Result<Height, Ics04Error> {
+        match self
+            .client_processed_heights
+            .get(&(client_id.clone(), height))
+        {
+            Some(height) => Ok(*height),
+            None => Err(Ics04Error::processed_height_not_found(
+                client_id.clone(),
+                height,
+            )),
+        }
+    }
+
     fn channel_counter(&self) -> Result<u64, Ics04Error> {
         Ok(self.channel_ids_counter)
     }
+
+    fn max_expected_time_per_block(&self) -> Duration {
+        Duration::from_secs(10)
+    }
 }
 
 impl ChannelKeeper for MockContext {
@@ -798,6 +932,64 @@ impl ClientReader for MockContext {
         }
     }
 
+    /// Search for the lowest consensus state higher than `height`.
+    fn next_consensus_state(
+        &self,
+        client_id: &ClientId,
+        height: Height,
+    ) -> Result<Option<AnyConsensusState>, Ics02Error> {
+        let client_record = self
+            .clients
+            .get(client_id)
+            .ok_or_else(|| Ics02Error::client_not_found(client_id.clone()))?;
+
+        // Get the consensus state heights and sort them in ascending order.
+        let mut heights: Vec<Height> = client_record.consensus_states.keys().cloned().collect();
+        heights.sort();
+
+        // Search for next state.
+        for h in heights {
+            if h > height {
+                // unwrap should never happen, as the consensus state for h must exist
+                return Ok(Some(
+                    client_record.consensus_states.get(&h).unwrap().clone(),
+                ));
+            }
+        }
+        Ok(None)
+    }
+
+    /// Search for the highest consensus state lower than `height`.
+    fn prev_consensus_state(
+        &self,
+        client_id: &ClientId,
+        height: Height,
+    ) -> Result<Option<AnyConsensusState>, Ics02Error> {
+        let client_record = self
+            .clients
+            .get(client_id)
+            .ok_or_else(|| Ics02Error::client_not_found(client_id.clone()))?;
+
+        // Get the consensus state heights and sort them in descending order.
+        let mut heights: Vec<Height> = client_record.consensus_states.keys().cloned().collect();
+        heights.sort_by(|a, b| b.cmp(a));
+
+        // Search for previous state.
+        for h in heights {
+            if h < height {
+                // unwrap should never happen, as the consensus state for h must exist
+                return Ok(Some(
+                    client_record.consensus_states.get(&h).unwrap().clone(),
+                ));
+            }
+        }
+        Ok(None)
+    }
+
+    fn host_height(&self) -> Height {
+        self.latest_height
+    }
+
     fn client_counter(&self) -> Result<u64, Ics02Error> {
         Ok(self.client_ids_counter)
     }
@@ -855,6 +1047,24 @@ impl ClientKeeper for MockContext {
     fn increase_client_counter(&mut self) {
         self.client_ids_counter += 1
     }
+
+    fn store_update_time(&mut self, client_id: ClientId, height: Height) -> Result<(), Ics02Error> {
+        let _ = self
+            .client_processed_times
+            .insert((client_id, height), ChannelReader::host_timestamp(self));
+        Ok(())
+    }
+
+    fn store_update_height(
+        &mut self,
+        client_id: ClientId,
+        height: Height,
+    ) -> Result<(), Ics02Error> {
+        let _ = self
+            .client_processed_heights
+            .insert((client_id, height), ClientReader::host_height(self));
+        Ok(())
+    }
 }
 
 impl Ics18Context for MockContext {
@@ -887,12 +1097,12 @@ impl Ics18Context for MockContext {
 
 #[cfg(test)]
 mod tests {
-    use crate::ics24_host::identifier::ChainId;
+    use crate::core::ics24_host::identifier::ChainId;
     use crate::mock::context::MockContext;
     use crate::mock::host::HostType;
     use crate::prelude::*;
     use crate::Height;
-    use test_env_log::test;
+    use test_log::test;
 
     #[test]
     fn test_history_manipulation() {
