@@ -1,18 +1,17 @@
 //! Host chain types and methods, used by context mock.
 
-use crate::prelude::*;
-use core::convert::TryFrom;
-
-use tendermint::chain::Id as TMChainId;
+use tendermint::time::Time;
 use tendermint_testgen::light_block::TmLightBlock;
 use tendermint_testgen::{Generator, LightBlock as TestgenLightBlock};
+use time::OffsetDateTime;
 
-use crate::ics02_client::client_consensus::AnyConsensusState;
-use crate::ics02_client::header::AnyHeader;
-use crate::ics07_tendermint::consensus_state::ConsensusState as TMConsensusState;
-use crate::ics07_tendermint::header::Header as TMHeader;
-use crate::ics24_host::identifier::ChainId;
+use crate::clients::ics07_tendermint::consensus_state::ConsensusState as TMConsensusState;
+use crate::clients::ics07_tendermint::header::Header as TMHeader;
+use crate::core::ics02_client::client_consensus::AnyConsensusState;
+use crate::core::ics02_client::header::AnyHeader;
+use crate::core::ics24_host::identifier::ChainId;
 use crate::mock::header::MockHeader;
+use crate::prelude::*;
 use crate::timestamp::Timestamp;
 use crate::Height;
 
@@ -52,7 +51,7 @@ impl HostBlock {
         match chain_type {
             HostType::Mock => HostBlock::Mock(MockHeader {
                 height: Height::new(chain_id.version(), height),
-                timestamp: Timestamp::from_nanoseconds(1).unwrap(),
+                timestamp: Timestamp::now(),
             }),
             HostType::SyntheticTendermint => {
                 HostBlock::SyntheticTendermint(Box::new(Self::generate_tm_block(chain_id, height)))
@@ -61,10 +60,16 @@ impl HostBlock {
     }
 
     pub fn generate_tm_block(chain_id: ChainId, height: u64) -> TmLightBlock {
-        let mut block = TestgenLightBlock::new_default(height).generate().unwrap();
-        block.signed_header.header.chain_id = TMChainId::try_from(chain_id.to_string()).unwrap();
+        // Sleep is required otherwise the generator produces blocks with the
+        // same timestamp as two block can be generated per second.
+        let ten_millis = core::time::Duration::from_millis(1000);
+        std::thread::sleep(ten_millis);
+        let time: Time = OffsetDateTime::now_utc().try_into().unwrap();
+        let time = time.duration_since(Time::unix_epoch()).unwrap().as_secs();
 
-        block
+        TestgenLightBlock::new_default_with_time_and_chain_id(chain_id.to_string(), time, height)
+            .generate()
+            .unwrap()
     }
 }
 
