@@ -1,6 +1,6 @@
 use serde::Serialize;
 
-use crate::ics23_commitment::commitment::CommitmentProofBytes;
+use crate::core::ics23_commitment::commitment::CommitmentProofBytes;
 use crate::Height;
 use flex_error::define_error;
 
