@@ -1,40 +1,40 @@
 pub mod step;
 
 use alloc::collections::btree_map::BTreeMap as HashMap;
-use core::convert::TryFrom;
+
 use core::fmt::Debug;
 use core::time::Duration;
 
-use ibc::ics02_client::client_consensus::AnyConsensusState;
-use ibc::ics02_client::client_state::AnyClientState;
-use ibc::ics02_client::client_type::ClientType;
-use ibc::ics02_client::context::ClientReader;
-use ibc::ics02_client::error as client_error;
-use ibc::ics02_client::header::AnyHeader;
-use ibc::ics02_client::msgs::create_client::MsgCreateAnyClient;
-use ibc::ics02_client::msgs::update_client::MsgUpdateAnyClient;
-use ibc::ics02_client::msgs::upgrade_client::MsgUpgradeAnyClient;
-use ibc::ics02_client::msgs::ClientMsg;
-use ibc::ics03_connection::connection::{Counterparty, State as ConnectionState};
-use ibc::ics03_connection::error as connection_error;
-use ibc::ics03_connection::msgs::conn_open_ack::MsgConnectionOpenAck;
-use ibc::ics03_connection::msgs::conn_open_confirm::MsgConnectionOpenConfirm;
-use ibc::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
-use ibc::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
-use ibc::ics03_connection::msgs::ConnectionMsg;
-use ibc::ics03_connection::version::Version;
-use ibc::ics04_channel::context::ChannelReader;
-use ibc::ics18_relayer::context::Ics18Context;
-use ibc::ics18_relayer::error as relayer_error;
-use ibc::ics23_commitment::commitment::{CommitmentPrefix, CommitmentProofBytes};
-use ibc::ics24_host::identifier::{ChainId, ClientId, ConnectionId};
-use ibc::ics26_routing::error as routing_error;
-use ibc::ics26_routing::msgs::Ics26Envelope;
+use ibc::core::ics02_client::client_consensus::AnyConsensusState;
+use ibc::core::ics02_client::client_state::AnyClientState;
+use ibc::core::ics02_client::client_type::ClientType;
+use ibc::core::ics02_client::context::ClientReader;
+use ibc::core::ics02_client::error as client_error;
+use ibc::core::ics02_client::header::AnyHeader;
+use ibc::core::ics02_client::msgs::create_client::MsgCreateAnyClient;
+use ibc::core::ics02_client::msgs::update_client::MsgUpdateAnyClient;
+use ibc::core::ics02_client::msgs::upgrade_client::MsgUpgradeAnyClient;
+use ibc::core::ics02_client::msgs::ClientMsg;
+use ibc::core::ics03_connection::connection::{Counterparty, State as ConnectionState};
+use ibc::core::ics03_connection::error as connection_error;
+use ibc::core::ics03_connection::msgs::conn_open_ack::MsgConnectionOpenAck;
+use ibc::core::ics03_connection::msgs::conn_open_confirm::MsgConnectionOpenConfirm;
+use ibc::core::ics03_connection::msgs::conn_open_init::MsgConnectionOpenInit;
+use ibc::core::ics03_connection::msgs::conn_open_try::MsgConnectionOpenTry;
+use ibc::core::ics03_connection::msgs::ConnectionMsg;
+use ibc::core::ics03_connection::version::Version;
+use ibc::core::ics04_channel::context::ChannelReader;
+use ibc::core::ics23_commitment::commitment::{CommitmentPrefix, CommitmentProofBytes};
+use ibc::core::ics24_host::identifier::{ChainId, ClientId, ConnectionId};
+use ibc::core::ics26_routing::error as routing_error;
+use ibc::core::ics26_routing::msgs::Ics26Envelope;
 use ibc::mock::client_state::{MockClientState, MockConsensusState};
 use ibc::mock::context::MockContext;
 use ibc::mock::header::MockHeader;
 use ibc::mock::host::HostType;
 use ibc::proofs::{ConsensusProof, Proofs};
+use ibc::relayer::ics18_relayer::context::Ics18Context;
+use ibc::relayer::ics18_relayer::error as relayer_error;
 use ibc::signer::Signer;
 use ibc::timestamp::ZERO_DURATION;
 use ibc::Height;
@@ -155,7 +155,7 @@ impl IbcTestRunner {
     }
 
     pub fn client_state(height: Height) -> AnyClientState {
-        AnyClientState::Mock(MockClientState(Self::mock_header(height)))
+        AnyClientState::Mock(MockClientState::new(Self::mock_header(height)))
     }
 
     pub fn consensus_state(height: Height) -> AnyConsensusState {
@@ -347,7 +347,7 @@ impl IbcTestRunner {
                 // create ICS26 message and deliver it
                 let msg = Ics26Envelope::Ics2Msg(ClientMsg::UpgradeClient(MsgUpgradeAnyClient {
                     client_id: Self::client_id(client_id),
-                    client_state: MockClientState(MockHeader::new(header)).into(),
+                    client_state: MockClientState::new(MockHeader::new(header)).into(),
                     consensus_state: MockConsensusState::new(MockHeader::new(header)).into(),
                     proof_upgrade_client: MerkleProof::try_from(c_bytes).unwrap(),
                     proof_upgrade_consensus_state: MerkleProof::try_from(cs_bytes).unwrap(),
@@ -471,6 +471,7 @@ impl modelator::step_runner::StepRunner<Step> for IbcTestRunner {
     }
 
     fn next_step(&mut self, step: Step) -> Result<(), String> {
+        let show = step.action.clone();
         let result = self.apply(step.action);
         let outcome_matches = match step.action_outcome {
             ActionOutcome::None => panic!("unexpected action outcome"),
@@ -529,7 +530,7 @@ impl modelator::step_runner::StepRunner<Step> for IbcTestRunner {
         }
 
         if !outcome_matches {
-            return Err("Action outcome did not match expected".into());
+            return Err(format!("Action outcome did not match expected: {:?}", show));
         }
 
         if !self.check_chain_states(step.chains) {
